<?php

namespace App\Security;

use App\Entity\User;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Mailer\Exception\TransportExceptionInterface;
use Symfony\Component\Mailer\MailerInterface;
use SymfonyCasts\Bundle\VerifyEmail\Exception\VerifyEmailExceptionInterface;
use SymfonyCasts\Bundle\VerifyEmail\VerifyEmailHelperInterface;

/**
 * The email verifier service
 */
class EmailVerifier {
    /**
     * @var VerifyEmailHelperInterface The email verifier service (from "VerifyEmail" bundle)
     */
    private readonly VerifyEmailHelperInterface $verifyEmailHelper;
    /**
     * @var MailerInterface The mailing service
     */
    private readonly MailerInterface $mailer;
    /**
     * @var EntityManagerInterface The entity manager
     */
    private readonly EntityManagerInterface $entityManager;

    /**
     * Initialisation
     *
     * @param VerifyEmailHelperInterface $verifyEmailHelper The email verifier service (from "VerifyEmail" bundle)
     * @param MailerInterface            $mailer            The mailing service
     * @param EntityManagerInterface     $entityManager     The entity manager
     */
    public function __construct (
        VerifyEmailHelperInterface $verifyEmailHelper,
        MailerInterface            $mailer,
        EntityManagerInterface     $entityManager
    ) {
        $this->verifyEmailHelper = $verifyEmailHelper;
        $this->mailer = $mailer;
        $this->entityManager = $entityManager;
    }

    /**
     * Send the confirmation email
     *
     * @param string         $verifyEmailRouteName The route managing the email verification
     * @param User           $user                 The user to confirm email
     * @param TemplatedEmail $email                The email itself
     *
     * @return void
     *
     * @throws TransportExceptionInterface
     */
    public function sendEmailConfirmation (string $verifyEmailRouteName, User $user, TemplatedEmail $email): void {
        $signatureComponents = $this->verifyEmailHelper->generateSignature(
            $verifyEmailRouteName,
            $user->getId(),
            $user->getEmail(),
            [
                'id' => $user->getId(),
            ]
        );

        $context = $email->getContext();
        $context['signedUrl'] = $signatureComponents->getSignedUrl();
        $context['expiresAtMessageKey'] = $signatureComponents->getExpirationMessageKey();
        $context['expiresAtMessageData'] = $signatureComponents->getExpirationMessageData();
        $email->context($context);

        $this->mailer->send($email);
    }

    /**
     * Handle the email confirmation
     *
     * @param Request $request The request
     * @param User    $user    The user
     *
     * @return void
     *
     * @throws VerifyEmailExceptionInterface If the confirmation link is invalid
     *
     * @noinspection PhpDocRedundantThrowsInspection
     */
    public function handleEmailConfirmation (Request $request, User $user): void {
        $this->verifyEmailHelper->validateEmailConfirmationFromRequest(
            $request,
            $user->getId(),
            $user->getEmail()
        );

        $user->setVerified(true);
        $this->entityManager->persist($user);
        $this->entityManager->flush();
    }
}
